package org.freecompany.brimstone.cache;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.CacheRequest;
import java.net.URLConnection;
import java.util.List;
import java.util.Map;

class ResourceCacheRequest extends CacheRequest {
    private File location;
    private Map< String, List< String>> headers;

    public ResourceCacheRequest( File location, URLConnection connection) {
        this.location = location;
        headers = connection.getHeaderFields();
    }

    public OutputStream getBody() throws IOException {
		SecurityManager sm = System.getSecurityManager();
		if ( sm != null) sm.checkPermission( new CachePermission( "cacheWrite"));

		try {
			return java.security.AccessController.doPrivileged( new java.security.PrivilegedExceptionAction< OutputStream>() {
					public OutputStream run() throws IOException {
						File dir = location.getParentFile();
						if ( !dir.isDirectory()) dir.mkdirs();

						File headersFile = ResourceCache.headersFile( location);
						if ( headers.isEmpty()) {
							headersFile.delete();
						} else {
							PrintWriter out = new PrintWriter( new FileOutputStream( headersFile));
							List< String> nullEntry = headers.get( null);
							if ( nullEntry == null) {
								out.println();
							} else {
								out.println( nullEntry.get( 0));
							}

							for ( Map.Entry< String, List< String>> entry : headers.entrySet()) {
								String header = entry.getKey();
								if ( header == null) continue;

								for ( String value : entry.getValue()) {
									out.print( header.toLowerCase());
									out.print( ':');
									out.println( value);
								}
							}
							out.close();
						}

						return new FileOutputStream( location);
					}
					});
		} catch ( java.security.PrivilegedActionException e) {
			throw ( IOException) e.getException();
		}
    }

    public void abort() {
        location.delete();
    }
}
