package org.freecompany.brimstone.cache;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.InputStream;
import java.io.IOException;
import java.net.CacheResponse;
import java.net.URLConnection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

class ResourceCacheResponse extends CacheResponse {
    private File location;
	private Map< String, List< String>> headers;

    public ResourceCacheResponse( File location) { this.location = location; }

    public Map< String, List< String>> getHeaders() throws IOException {
		if ( headers != null) return headers;
        headers = new HashMap< String, List< String>>();
		readHeadersFile();
		if ( !headers.containsKey( "content-type")) {
			String type = URLConnection.guessContentTypeFromName( location.getName());
			if ( type == null) type = URLConnection.guessContentTypeFromStream( new FileInputStream( location));
			if ( type == null) type = "application/octet-stream";
			headers.put( "content-type", Collections.singletonList( type));
		}
		if ( !headers.containsKey( "content-length")) headers.put( "content-length", Collections.singletonList( Long.toString( location.length())));
        return headers;
    }

	private void readHeadersFile() throws IOException {
		try {
			java.security.AccessController.doPrivileged( new java.security.PrivilegedExceptionAction< Object>() {
					public Object run() throws IOException {
						try {
							BufferedReader in = new BufferedReader( new FileReader( ResourceCache.headersFile( location)));
							String s = in.readLine();
							if ( !"".equals( s)) headers.put( null, Collections.singletonList( s));

							while ( true) {
								s = in.readLine();
								if ( s == null) break;

								int pos = s.indexOf( ':');
								if ( pos < 0) continue;

								String header = s.substring( 0, pos);
								List< String> list = headers.get( header);
								if ( list == null) headers.put( header, list = new LinkedList< String>());
								list.add( s.substring( pos + 1));
							}
							in.close();
						} catch( FileNotFoundException e) {
							/**
							 * TODO: Possibly have a callback for Imp to insert entries into 
							 * the cache with the proper header data.
							 */
							assert true;
						}
						return null;
					}
					});
		} catch ( java.security.PrivilegedActionException e) {
			throw ( IOException) e.getException();
		}
	}

    public InputStream getBody() throws IOException {
		try {
			return java.security.AccessController.doPrivileged( new java.security.PrivilegedExceptionAction< InputStream>() {
					public InputStream run() throws IOException {
						return new FileInputStream( location);
					}
					});
		} catch ( java.security.PrivilegedActionException e) {
			throw ( IOException) e.getException();
		}
    }
}
